static char rcsid[] = "$Header: /home/cvsroot/xmimonc/imonc.c,v 1.1.1.1 2001/11/17 16:30:04 kelly Exp $";
/*
 * $Header: /home/cvsroot/xmimonc/imonc.c,v 1.1.1.1 2001/11/17 16:30:04 kelly Exp $
 *	
 * (c) 2001 Andreas Keldenich <kelly@myfoo.de>
 *
 * $Log: imonc.c,v $
 * Revision 1.1.1.1  2001/11/17 16:30:04  kelly
 * Initial revision
 *
 */

#include <stdio.h>
#include <stdarg.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include <Xm/Xm.h>

#include "xmimonc.h"


static char buffer[BUFSIZ];
static char response[BUFSIZ];
static int fd;


static void ImoncRecv(XtPointer client_data, int *fd, XtInputId *id)
{
  int c = 0;
  
  c = recv(*fd, buffer, BUFSIZ-1, 0);
  if (c == -1)
  {
    XmiStatus("Read error: %s", strerror(errno));
    return;
  }
  
  printf("%s", buffer);
  
}


static int ImoncSend(char *s)
{
  int count;
  int len;
  
  len = strlen(s);
  
  count = send(fd, s, len, 0);
  if (count < len)
  {
    XmiStatus("Send error: %s", strerror(errno));
    return -1;
  }
  return count;
}


static int ImoncRead(char *buffer, int len)
{
  int count;
  
  count = recv(fd, buffer, len, 0);
  if (count == -1)
  {
    XmiStatus("Read error: %s", strerror(errno));
  }
  return count;
}


static int ImoncReadLn(char *line, int max)
{
  int i = 0;
  
  while (i < (max - 1))
  {
    ImoncRead(line+i, 1);
    if (line[i] == '\r') /* ignore \r */
      continue;
    if (line[i++] == '\n')
    {
      line[--i] = '\0';
      break;
    }
  }
  return i;
}


char *ImoncCommand(char *format, ...)
{
  int len = 0;
  char *s;
  va_list ap;
  char cmd[200];
  XmString string;
  
  va_start(ap, format);
  vsprintf(cmd, format, ap);
  va_end(ap);
  
  /* send command */
  if (ImoncSend(cmd) < 0)
    return NULL;
  if (ImoncSend("\r\n") < 0)
    return NULL;
  
  /* read the response */
  bzero(response, BUFSIZ);
  len = ImoncReadLn(response, BUFSIZ);
  
  if (len < 3)
  {
    return NULL;
  }
  
  if (! strncmp(response, "OK ", 3))
  {
    s = response + 3;
    return s;
  }
  
  return NULL;
}


static void ImoncUpdate(XtPointer client_data, XtIntervalId *id)
{
  FormData *form_data = (FormData *) client_data;
  char *s;
  long sec = time(NULL);
  static long old_sec = 0;
  int n = 0, i, online = 0, rate = 0;
  int bi = 0, bo = 0, bytes_total = 0, tbi = 0, tbo = 0;
  char ip[20] = " ";
  XmString string;
  
  s = ImoncCommand("cpu", i);
  if (s)
    UtilUpdateLabel(form_data->label_cpu_data, "%s%%", s);
  
  if (old_sec != sec)
  {
    for (i = 1; i <= form_data->channel_count; i++)
    {
      s = ImoncCommand("status %d", i);
      if (s)
      {
        if (! strcmp(s, "Online"))
        {
	  online++;
	  form_data->channels[i-1].status = 1;
        }
	form_data->channels[i-1].status = 0;
	s = ImoncCommand("rate %d", i);
        if (s)
        {
	  n = sscanf(s, "%d %d", &bi, &bo);
	  form_data->channels[i-1].bi = bi;
	  form_data->channels[i-1].bo = bo;
	  bytes_total += bi;
	  bytes_total += bo;
	  tbi += bi;
	  tbo += bo;
	  
        }
        s = ImoncCommand("ip %d", i);
        if (s)
        {
	  strcpy(form_data->channels[i-1].ip, s);
	  strcpy(ip, s);
        }
      } /* if online */
    } /* loop over channels */
    
    LoadStore(tbi, tbo);
    LoadUpdate(form_data);
    
    /* Update the display
     */
    if (online)
    {
      XmiStatusLed(STATUS_ONLINE);
      XmiStatus("Online");
      XmiSetTitle("Online");
    }
    else
    {
      XmiStatusLed(STATUS_OFFLINE);
      XmiStatus("Offline");
      XmiSetTitle("Offline");
    }
    
    UtilUpdateLabel(form_data->label_bytes_data, "%d", bytes_total);
    UtilUpdateLabel(form_data->label_ip_data, ip);
    
    old_sec = sec;
  }
  XtAppAddTimeOut(form_data->app, 50, ImoncUpdate, form_data);  
}


static int ImoncConnect(FormData *form_data)
{
  struct hostent *hp;
  struct servent *sp;
  struct sockaddr_in remote;
  int port = PORT;
  char *server = HOST;
  char *s;
  
  XmiStatus("Connet...");
  XmiStatusLed(STATUS_NO_CONNECT);
  
  fd = socket(AF_INET, SOCK_STREAM, 0);
  if (fd == -1)
  {
    XmiStatus("Connet failed: socket: %s", strerror(errno));
    return -1;
  }
  
  /*  get the host by name  */
  bzero((char*) &remote, sizeof(remote));
  remote.sin_family = AF_INET;
  remote.sin_port = htons ((unsigned short) port);
  hp = gethostbyname(server);
  if (hp == NULL)
  {
    XmiStatus("Connect failed: Can't rsolve host name: %s", server);
    return -1;
  }
  remote.sin_addr.s_addr = ((struct in_addr *) (hp->h_addr))->s_addr;
  
  /* connect to host */
  if (connect(fd, (struct sockaddr*) &remote, sizeof(remote)) == -1)
  {
    XmiStatus("Connect failed: connect: %s", strerror(errno));
    return -1;
  }

#ifdef	USE_CALLBACK
  /* add the input callback and the time out timer  */
  form_data->cbid = XtAppAddInput(form_data->app, fd, (XtPointer) XtInputReadMask, ImoncRecv, NULL);
#endif

  form_data->fd = fd;
  XmiStatus("Connect to: %s", server);
  XmiStatusLed(STATUS_UNKNOWN);
  
  s = ImoncCommand("channels");
  if (s)
  {
    form_data->channel_count = atoi(s);
    form_data->channels = (Channel *) malloc(form_data->channel_count * sizeof(Channel));
    if (! form_data->channels)
      exit(-1);
  }
  XtAppAddTimeOut(form_data->app, 100, ImoncUpdate, form_data);
  
  return fd;
}


void ImoncTimeOutConnect(XtPointer client_data, XtIntervalId *id)
{
  FormData *form_data = (FormData *) client_data;
  static int retry = 0;
  
  if (ImoncConnect(form_data) < 0)
  {
    retry++;
    if (retry < MAX_CONNECT_RETRY)
    {
      /* try again in two seconds. */
      XtAppAddTimeOut(form_data->app, 2000, ImoncTimeOutConnect, form_data); 
    }
    else
      XmiStatus("Giving up - can not connect :-(");
  }
  
}


int ImoncDisconnect(FormData *form_data)
{
#ifdef	USE_CALLBACK
  XtRemoveInput(form_data->cbid);
#endif
    
  if (close(form_data->fd) < 0)
    return -1;
  return 0;
}
