static char rcsid[] = "$Header: /home/cvsroot/xmimonc/main.c,v 1.1.1.1 2001/11/17 16:30:04 kelly Exp $";
/*
 * $Header: /home/cvsroot/xmimonc/main.c,v 1.1.1.1 2001/11/17 16:30:04 kelly Exp $
 *	
 * (c) 2001 Andreas Keldenich <kelly@myfoo.de>
 *
 * $Log: main.c,v $
 * Revision 1.1.1.1  2001/11/17 16:30:04  kelly
 * Initial revision
 *
 */

#include <stdarg.h>

#include <Xm/Xm.h>
#include <Xm/MwmUtil.h>
#include <Xm/Protocols.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/RowColumn.h>
#include <Xm/DrawingA.h>
#include <Xm/Label.h>
#include <Xm/Separator.h>
#include <Xm/ToggleB.h>
#include <Xm/PushB.h>
#include <Xm/CascadeBG.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>


#include "xmimonc.h"



/* include the fallback resources */
String xmi_fallbacks[] =
{
#include "XMimonc.ad.h"
NULL,
};


static FormData form_data;

static MenuItem file_menu[] =
{
  {"exit", 		NULL, &xmPushButtonWidgetClass, 0, NULL, NULL, XmiExit, &form_data, (MenuItem *) NULL},
  NULL,
};


void XmiExit(Widget w, XtPointer client_data, XtPointer call_data)
{
  FormData *form_data = (FormData *) client_data;
  
  ImoncDisconnect(form_data);
  
  /* check if it is ok to close... */
  exit(0);
}


void XmiStatus(char *format, ...)
{
  va_list ap;
  char s[200];
  XmString string;
  
  va_start(ap, format);
  vsprintf(s, format, ap);
  va_end(ap);
  string = XmStringCreateLtoR(s, XmFONTLIST_DEFAULT_TAG);
  XtVaSetValues(form_data.label_status,
  	XmNlabelString,		string,
    	NULL);
  XmUpdateDisplay(form_data.label_status); 
  XmStringFree(string);
}


void XmiStatusLed(int what)
{
  Pixmap image = form_data.unknown;
  
  switch (what)
  {
    case STATUS_OFFLINE :
      image = form_data.offline;
      break;
    case STATUS_ONLINE :
      image = form_data.online;
      break;
    case STATUS_NO_CONNECT :
      image = form_data.no_connect;
      break;
    default :
      break;
  }
  XtVaSetValues(form_data.label_status_led,
  	XmNlabelPixmap,		image,
	NULL);
  XmUpdateDisplay(form_data.label_status_led); 
}


void XmiSetTitle(char *s)
{
  char title[100];
  char icon_name[100];
  
  sprintf(title, "XMimonc - %s", s);
  sprintf(icon_name, "XMimonc - %s", s);
  XtVaSetValues(form_data.app_shell, 
  	XmNtitle,	title,
  	XmNiconName,	icon_name,
  	NULL);
}


/*
 * author:	Kelly
 * date:	11.08.95
 * returns:	The menu widget.
 */
static Widget MenuBuild(Widget parent,
			int menu_type,
			char *menu_title, 
			char menu_mnemonic,
			Boolean tear_off,
			MenuItem *items)
{
  Widget menu, cascade, widget;
  int i;
  XmString str;
  
  if (menu_type == XmMENU_PULLDOWN || menu_type == XmMENU_OPTION)
    menu = XmCreatePulldownMenu(parent, "_pulldown", NULL, 0);
  else 
    if (menu_type == XmMENU_POPUP)
      menu = XmCreatePopupMenu(parent, "_popup", NULL, 0);
    else 
    {
      XtWarning("Invalid menu type passed to MenuBuild()");
      return NULL;
    }
  if (tear_off)
    XtVaSetValues(menu, XmNtearOffModel, XmTEAR_OFF_ENABLED, NULL);
  
  /* Pulldown menus require a cascade button to be made */
  if (menu_type == XmMENU_PULLDOWN) 
  {
    str = XmStringCreateLocalized (menu_title);
    cascade = XtVaCreateManagedWidget(menu_title,
    		xmCascadeButtonGadgetClass, 	parent,
    		XmNsubMenuId,   		menu,
    		XmNlabelString, 		str,
    		XmNmnemonic,    		menu_mnemonic,
    		NULL);
    XmStringFree (str);
  } 
  else 
    if (menu_type == XmMENU_OPTION) 
    {
      /* Option menus are a special case, but not hard to handle */
      Arg args[5];
      int n = 0;
      str = XmStringCreateLocalized(menu_title);
      XtSetArg(args[n], XmNsubMenuId,	menu);	n++;
      XtSetArg(args[n], XmNlabelString,	str);	n++;
      /* This really isn't a cascade, but this is the widget handle
       * we're going to return at the end of the function.
       */
      cascade = XmCreateOptionMenu(parent, menu_title, args, n);
      XmStringFree(str);
    }

  /* Now add the menu items */
  for (i = 0; items[i].label != NULL; i++) 
  {
    /* If subitems exist, create the pull-right menu by calling this
     * function recursively.  Since the function returns a cascade
     * button, the widget returned is used..
     */
    if (items[i].subitems)
      if (menu_type == XmMENU_OPTION) 
      {
        XtWarning("You can't have submenus from option menu items.");
        continue;
      } 
      else
        widget = MenuBuild(menu, 
        		XmMENU_PULLDOWN, 
        		items[i].label, 
        		items[i].mnemonic, 
        		tear_off, 
        		items[i].subitems);
    else
      widget = XtVaCreateManagedWidget(items[i].label,
      			*items[i].class, 	menu,
      			NULL);
    
    /* Whether the item is a real item or a cascade button with a
     * menu, it can still have a mnemonic.
     */
    if (items[i].mnemonic)
      XtVaSetValues(widget, XmNmnemonic, items[i].mnemonic, NULL);
    
    /* any item can have an accelerator, except cascade menus. But,
     * we don't worry about that; we know better in our declarations.
     */
    if (items[i].accelerator) 
    {
      str = XmStringCreateLocalized(items[i].accel_text);
      XtVaSetValues(widget,
      			XmNaccelerator, 	items[i].accelerator,
                	XmNacceleratorText, 	str,
                	NULL);
      XmStringFree(str);
    }
    
    if (items[i].callback)
       XtAddCallback(widget,
                items[i].class == &xmToggleButtonWidgetClass ||
                items[i].class == &xmToggleButtonGadgetClass ?
                	XmNvalueChangedCallback : /* ToggleButton class */
                	XmNactivateCallback,      /* PushButton class */
                items[i].callback, 
                items[i].callback_data);
    items[i].widget = widget;
  }

  /* for popup menus, just return the menu; pulldown menus, return
   * the cascade button; option menus, return the thing returned
   * from XmCreateOptionMenu().  This isn't a menu, or a cascade button!
   */
  return menu_type == XmMENU_POPUP ? menu : cascade;
}


static void XmiCreateMainWindow(FormData *form_data)
{
  Arg args[10];
  Cardinal n = 0;
  Widget help;
  Pixel fg, bg;
  Pixmap image;
  
  /* create form */
  form_data->form = XtVaCreateManagedWidget("form",
  	xmFormWidgetClass,
  	form_data->app_shell,
  	NULL);
  
  /* create menu */
  n = 0;
  XtSetArg(args[n], XmNtopAttachment, 	XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNleftAttachment, 	XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNrightAttachment,	XmATTACH_FORM); n++;
  form_data->menu = XmCreateMenuBar(form_data->form, "xmiMenu", args, n);
  MenuBuild(form_data->menu, XmMENU_PULLDOWN, "File", 		'F', False, file_menu);
  /*
  help = MenuBuild(form_data->menu, XmMENU_PULLDOWN, "Help", 	'H', False, help_menu);
  XtVaSetValues(form_data->menu, XmNmenuHelpWidget, help, NULL);
  */
  XtManageChild(form_data->menu);
  
  /* build status line a load pixmaps...
   */
  XtVaGetValues(form_data->form, 
  		XmNforeground, &fg,
		XmNbackground, &bg,
  		NULL);
  image = XmGetPixmap(XtScreen(form_data->form), "online.xpm", fg, bg);
  if (image == XmUNSPECIFIED_PIXMAP)
    XtWarning("Failed to load online image.");
  else
    form_data->online = image;
  image = XmGetPixmap(XtScreen(form_data->form), "offline.xpm", fg, bg);
  if (image == XmUNSPECIFIED_PIXMAP)
    XtWarning("Failed to load offline image.");
  else
    form_data->offline = image;
  image = XmGetPixmap(XtScreen(form_data->form), "noconnect.xpm", fg, bg);
  if (image == XmUNSPECIFIED_PIXMAP)
    XtWarning("Failed to load no_connect image.");
  else
    form_data->no_connect = image;
  image = XmGetPixmap(XtScreen(form_data->form), "unknown.xpm", fg, bg);
  if (image == XmUNSPECIFIED_PIXMAP)
    XtWarning("Failed to load unknown image.");
  else
    form_data->unknown = image;
  
  form_data->frame_status_led = XtVaCreateManagedWidget("frameStatusLed",
  	xmFrameWidgetClass,
  	form_data->form,
	XmNshadowType,			XmSHADOW_IN,
	XmNchildType,			XmFRAME_WORKAREA_CHILD,
  	XmNrightAttachment,		XmATTACH_FORM,
  	XmNrightOffset,			3,
  	XmNbottomAttachment,		XmATTACH_FORM,
  	XmNbottomOffset,		3,
  	XmNalignment,			XmALIGNMENT_BEGINNING,
	XmNshadowThickness,		1,
	NULL);
  
  form_data->label_status_led = XtVaCreateManagedWidget("labelStatusLed",
  	xmLabelWidgetClass,
  	form_data->frame_status_led,
	XmNalignment,			XmALIGNMENT_CENTER,
	XmNlabelType,			XmPIXMAP,
	XmNlabelPixmap,			form_data->unknown,
  	NULL);
  
  form_data->frame_status = XtVaCreateManagedWidget("frameStatus",
  	xmFrameWidgetClass,
  	form_data->form,
	XmNshadowType,			XmSHADOW_IN,
	XmNchildType,			XmFRAME_WORKAREA_CHILD,
	XmNleftAttachment,		XmATTACH_FORM,
  	XmNleftOffset,			3,
  	XmNrightAttachment,		XmATTACH_WIDGET,
	XmNrightWidget,			form_data->frame_status_led,
  	XmNrightOffset,			3,
  	XmNbottomAttachment,		XmATTACH_FORM,
  	XmNbottomOffset,		3,
  	XmNalignment,			XmALIGNMENT_BEGINNING,
	XmNshadowThickness,		1,
	NULL);
  
  form_data->label_status = XtVaCreateManagedWidget("labelStatus",
  	xmLabelWidgetClass,
  	form_data->frame_status,
	XmNalignment,			XmALIGNMENT_BEGINNING,
	XmNrecomputeSize,		False,
  	NULL);
  
  form_data->sep = XtVaCreateManagedWidget("sep",
  	xmSeparatorWidgetClass,
  	form_data->form,
  	XmNbottomAttachment,		XmATTACH_WIDGET,
  	XmNbottomWidget,		form_data->frame_status,
  	XmNbottomOffset,		2, 
  	XmNleftAttachment,		XmATTACH_FORM,
  	XmNrightAttachment,		XmATTACH_FORM,
  	NULL);
	
  form_data->rc_labels = XtVaCreateManagedWidget("rcLabels",
  	xmRowColumnWidgetClass,
  	form_data->form,
  	XmNtopAttachment,		XmATTACH_WIDGET,
  	XmNtopWidget,			form_data->menu,
  	XmNtopOffset,			1,
  	XmNleftAttachment,		XmATTACH_FORM,
  	XmNleftOffset,			1,
  	XmNpacking,			XmPACK_COLUMN,
	XmNnumColumns,			1,
	XmNisAligned,			True,
	XmNentryAlignment,		XmALIGNMENT_BEGINNING,
  	NULL);
  
  form_data->rc_data = XtVaCreateManagedWidget("rcData",
  	xmRowColumnWidgetClass,
  	form_data->form,
  	XmNtopAttachment,		XmATTACH_WIDGET,
  	XmNtopWidget,			form_data->menu,
  	XmNtopOffset,			1,
  	XmNleftAttachment,		XmATTACH_WIDGET,
	XmNleftWidget,			form_data->rc_labels,
  	XmNleftOffset,			1,
  	XmNpacking,			XmPACK_COLUMN,
	XmNnumColumns,			1,
	XmNisAligned,			True,
	XmNentryAlignment,		XmALIGNMENT_END,
  	NULL);
  
  help = XtVaCreateManagedWidget(" ",
  	xmLabelWidgetClass,
  	form_data->form,
  	XmNtopAttachment,		XmATTACH_WIDGET,
  	XmNtopWidget,			form_data->rc_data,
  	XmNtopOffset,			1,
  	XmNbottomAttachment,		XmATTACH_WIDGET,
  	XmNbottomWidget,		form_data->sep,
  	XmNbottomOffset,		0, 
  	XmNleftAttachment,		XmATTACH_FORM,
  	XmNleftOffset,			1,
  	NULL);
	
  form_data->label_bytes = XtVaCreateManagedWidget("labelBytes",
  	xmLabelWidgetClass,
  	form_data->rc_labels,
	XmNalignment,			XmALIGNMENT_BEGINNING,
	XmNrecomputeSize,		True, 
  	NULL);
  	
  form_data->label_bytes_data = XtVaCreateManagedWidget("labelBytesData",
  	xmLabelWidgetClass,
  	form_data->rc_data,
	XmNalignment,			XmALIGNMENT_END,
	XmNrecomputeSize,		False, 
  	NULL);
  
  form_data->label_ip = XtVaCreateManagedWidget("labelIp",
  	xmLabelWidgetClass,
  	form_data->rc_labels,
	XmNalignment,			XmALIGNMENT_BEGINNING,
	XmNrecomputeSize,		True, 
  	NULL);
  	
  form_data->label_ip_data = XtVaCreateManagedWidget("labelIpData",
  	xmLabelWidgetClass,
  	form_data->rc_data,
	XmNalignment,			XmALIGNMENT_END,
	XmNrecomputeSize,		False, 
  	NULL);
  
  form_data->label_cpu = XtVaCreateManagedWidget("labelCpu",
  	xmLabelWidgetClass,
  	form_data->rc_labels,
	XmNalignment,			XmALIGNMENT_BEGINNING,
	XmNrecomputeSize,		True, 
  	NULL);
  	
  form_data->label_cpu_data = XtVaCreateManagedWidget("labelCpuData",
  	xmLabelWidgetClass,
  	form_data->rc_data,
	XmNalignment,			XmALIGNMENT_END,
	XmNrecomputeSize,		False, 
  	NULL);
  
  form_data->frame_load = XtVaCreateManagedWidget("frameLoad",
  	xmFrameWidgetClass,
  	form_data->form,
  	XmNleftAttachment,		XmATTACH_WIDGET,
	XmNleftWidget,			form_data->rc_data,
  	XmNleftOffset,			3,
  	XmNtopAttachment,		XmATTACH_WIDGET,
  	XmNtopWidget,			form_data->menu,
  	XmNtopOffset,			3,
  	XmNbottomAttachment,		XmATTACH_WIDGET,
  	XmNbottomWidget,		form_data->sep,
  	XmNbottomOffset,		2, 
  	XmNrightAttachment,		XmATTACH_FORM,
  	XmNrightOffset,			3,
	XmNshadowType,			XmSHADOW_IN,
	XmNchildType,			XmFRAME_WORKAREA_CHILD,
	XmNshadowThickness,		1,
	NULL);
  
  form_data->da_load = XtVaCreateManagedWidget("daLoad",
  	xmDrawingAreaWidgetClass,
  	form_data->frame_load,
	NULL);
  LoadInit(form_data->da_load);
  XtAddCallback(form_data->da_load, XmNexposeCallback, LoadExposeCB, form_data);
  XtAddCallback(form_data->da_load, XmNresizeCallback, LoadResizeCB, form_data);
}


/*
 * Author:	Kelly
 * Date:	Sat May 25 12:52:28 MET DST 1996
 * Description:	
 */
int main(int argc, char *argv[], char *envv[])
{
  XEvent event;
  int decor, func;

  XtSetLanguageProc(form_data.app, NULL, NULL);
  form_data.app_shell = XtVaAppInitialize(&form_data.app, APPCLASS, 
  	NULL, 0, &argc, argv, xmi_fallbacks, 
  	NULL);
  XmiCreateMainWindow(&form_data);
  decor = MWM_DECOR_TITLE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU | MWM_DECOR_BORDER;
  func = MWM_FUNC_MOVE | MWM_FUNC_MINIMIZE | MWM_FUNC_CLOSE;
  XtVaSetValues(form_data.app_shell, 
  	/* XmNmwmDecorations,	decor,
  	XmNmwmFunctions,	func, */
  	XmNdeleteResponse,	XmDO_NOTHING,
  	XmNallowShellResize, 	True, 
	XmNminWidth,		250,
	XmNminHeight,		150,
  	NULL);
  XmAddWMProtocolCallback(form_data.app_shell, 
  	XInternAtom(XtDisplay(form_data.app_shell), 
		"WM_DELETE_WINDOW", 
		True), 
	XmiExit, 
	&form_data); 
  IconWindow(&form_data);
  XtRealizeWidget(form_data.app_shell);
  XmiSetTitle("not connected");
  XtAppAddTimeOut(form_data.app, 200, ImoncTimeOutConnect, &form_data); 
  for (;;)				/* Event loop			*/
  {					/* This replaces XtMainLoop 	*/
    XtAppNextEvent(form_data.app, &event);
    XtDispatchEvent(&event);
    /* if (event.type == Expose)
      XccsIconRedraw(); */
  }
}
